@@ -82,16 +82,18 @@ class PixelMeasuresSequence(DataElementSequence):
 
     def __init__(
             self,
-            pixel_spacing: Tuple[float, float],
+            pixel_spacing: Sequence[float],
             slice_thickness: float,
             spacing_between_slices: Optional[float] = None,
         ) -> None:
         """
         Parameters
         ----------
-        pixel_spacing: Tuple[float, float]
+        pixel_spacing: Sequence[float]
             Distance in physical space between neighboring pixels in
-            millimeters along the row and column dimension of the image
+            millimeters along the row and column dimension of the image. First
+            value represents the spacing between rows (vertical) and second
+            value represents the spacing between columns (horizontal).
         slice_thickness: float
             Depth of physical space volume the image represents in millimeter
         spacing_between_slices: float, optional
@@ -117,36 +119,46 @@ class PlanePositionSequence(DataElementSequence):
     """
 
     def __init__(
-            self,
-            coordinate_system: Union[str, CoordinateSystemNames],
-            image_position: Tuple[float, float, float],
-            pixel_matrix_position: Optional[Tuple[int, int]] = None
-        ) -> None:
+        self,
+        coordinate_system: Union[str, CoordinateSystemNames],
+        image_position: Sequence[float],
+        pixel_matrix_position: Optional[Tuple[int, int]] = None
+    ) -> None:
         """
         Parameters
         ----------
-        image_position: Tuple[float, float, float]
+        image_position: Sequence[float]
             Offset of the first row and first column of the plane (frame) in
             millimeter along the x, y, and z axis of the three-dimensional
             patient or slide coordinate system
         pixel_matrix_position: Tuple[int, int], optional
-            Offset of the first row and first column of the plane (frame) in
+            Offset of the first column and first row of the plane (frame) in
             pixels along the row and column direction of the total pixel matrix
             (only required if `coordinate_system` is ``"SLIDE"``)
 
+        Note
+        ----
+        The values of both `image_position` and `pixel_matrix_position` are
+        one-based.
+
         """
         super().__init__()
         item = Dataset()
+
+        def ds(num: float) -> float:
+            return float(str(num)[:16])
+
         if coordinate_system == CoordinateSystemNames.SLIDE:
             if pixel_matrix_position is None:
                 raise TypeError(
                     'Position in Pixel Matrix must be specified for '
                     'slide coordinate system.'
                 )
-            row_position, col_position = pixel_matrix_position
-            item.XOffsetInSlideCoordinateSystem = image_position[0]
-            item.YOffsetInSlideCoordinateSystem = image_position[1]
-            item.ZOffsetInSlideCoordinateSystem = image_position[2]
+            col_position, row_position = pixel_matrix_position
+            x, y, z = image_position
+            item.XOffsetInSlideCoordinateSystem = ds(x)
+            item.YOffsetInSlideCoordinateSystem = ds(y)
+            item.ZOffsetInSlideCoordinateSystem = ds(z)
             item.RowPositionInTotalImagePixelMatrix = row_position
             item.ColumnPositionInTotalImagePixelMatrix = col_position
         elif coordinate_system == CoordinateSystemNames.PATIENT:
@@ -183,11 +195,15 @@ class PlanePositionSequence(DataElementSequence):
                     other[0].XOffsetInSlideCoordinateSystem,
                     other[0].YOffsetInSlideCoordinateSystem,
                     other[0].ZOffsetInSlideCoordinateSystem,
+                    other[0].RowPositionInTotalImagePixelMatrix,
+                    other[0].ColumnPositionInTotalImagePixelMatrix,
                 ]),
                 np.array([
                     self[0].XOffsetInSlideCoordinateSystem,
                     self[0].YOffsetInSlideCoordinateSystem,
                     self[0].ZOffsetInSlideCoordinateSystem,
+                    self[0].RowPositionInTotalImagePixelMatrix,
+                    self[0].ColumnPositionInTotalImagePixelMatrix,
                 ]),
             )
 
@@ -202,7 +218,7 @@ class PlaneOrientationSequence(DataElementSequence):
     def __init__(
             self,
             coordinate_system: Union[str, CoordinateSystemNames],
-            image_orientation: Tuple[float, float, float, float, float, float]
+            image_orientation: Sequence[float]
         ) -> None:
         """
         Parameters
@@ -210,9 +226,9 @@ class PlaneOrientationSequence(DataElementSequence):
         coordinate_system: Union[str, highdicom.enum.CoordinateSystemNames]
             Subject (``"PATIENT"`` or ``"SLIDE"``) that was the target of
             imaging
-        image_orientation: Tuple[float, float, float, float, float, float]
+        image_orientation: Sequence[float]
             Direction cosines for the first row (first triplet) and the first
-            column (second triplet) of an image with respect to the x, y, and z
+            column (second triplet) of an image with respect to the X, Y, and Z
             axis of the three-dimensional coordinate system
 
         """
