@@ -1,20 +1,21 @@
 """Module for the SOP class of the Segmentation IOD."""
-import itertools
 import logging
 import numpy as np
 from collections import defaultdict
 from typing import Any, Dict, List, Optional, Set, Sequence, Union, Tuple
 
 from pydicom.dataset import Dataset
+from pydicom.encaps import decode_data_sequence, encapsulate
 from pydicom.pixel_data_handlers.numpy_handler import pack_bits
 from pydicom.pixel_data_handlers.util import get_expected_length
-from pydicom.uid import UID
-from pydicom.sr.codedict import codes
-from pydicom._storage_sopclass_uids import (
-    SegmentationStorage,
-    VLSlideCoordinatesMicroscopicImageStorage,
-    VLWholeSlideMicroscopyImageStorage,
+from pydicom.uid import (
+    ExplicitVRLittleEndian,
+    ImplicitVRLittleEndian,
+    JPEG2000Lossless,
+    RLELossless,
+    UID,
 )
+from pydicom.sr.codedict import codes
 
 from highdicom.base import SOPClass
 from highdicom.content import (
@@ -23,6 +24,7 @@ from highdicom.content import (
     PixelMeasuresSequence
 )
 from highdicom.enum import CoordinateSystemNames
+from highdicom.frame import encode_frame
 from highdicom.seg.content import (
     DimensionIndexSequence,
     SegmentDescription,
@@ -33,7 +35,6 @@ from highdicom.seg.enum import (
     SegmentsOverlapValues,
 )
 from highdicom.sr.coding import CodedConcept
-from highdicom.utils import compute_plane_positions_tiled_full
 
 
 logger = logging.getLogger(__name__)
@@ -67,7 +68,7 @@ class Segmentation(SOPClass):
             max_fractional_value: int = 255,
             content_description: Optional[str] = None,
             content_creator_name: Optional[str] = None,
-            transfer_syntax_uid: Union[str, UID] = '1.2.840.10008.1.2',
+            transfer_syntax_uid: Union[str, UID] = ImplicitVRLittleEndian,
             pixel_measures: Optional[PixelMeasuresSequence] = None,
             plane_orientation: Optional[PlaneOrientationSequence] = None,
             plane_positions: Optional[Sequence[PlanePositionSequence]] = None,
@@ -83,20 +84,21 @@ class Segmentation(SOPClass):
             Array of segmentation pixel data of boolean, unsigned integer or
             floating point data type representing a mask image. If `pixel_array`
             is a floating-point array or a binary array (containing only the
-            values ``True`` and ``False`` or ``0`` and ``1``), the segment number
-            used to encode the segment is taken from segment_descriptions.
-            Otherwise, if pixel_array contains multiple integer values, each value
-            is treated as a different segment whose segment number is that integer
-            value. In this case, all segments found in the array must be described
-            in `segment_descriptions`. Note that this is valid for both ``"BINARY"``
-            and ``"FRACTIONAL"`` segmentations.
-            For ``"FRACTIONAL"`` segmentations, values either encode the probability
-            of a given pixel belonging to a segment
+            values ``True`` and ``False`` or ``0`` and ``1``), the segment
+            number used to encode the segment is taken from
+            `segment_descriptions`. Otherwise, if `pixel_array` contains
+            multiple integer values, each value is treated as a different
+            segment whose segment number is that integer value. In this case,
+            all segments found in the array must be described
+            in `segment_descriptions`. Note that this is valid for both
+            ``"BINARY"`` and ``"FRACTIONAL"`` segmentations.
+            For ``"FRACTIONAL"`` segmentations, values either encode the
+            probability of a given pixel belonging to a segment
             (if `fractional_type` is ``"PROBABILITY"``)
             or the extent to which a segment occupies the pixel
             (if `fractional_type` is ``"OCCUPANCY"``).
-            When `pixel_array` has a floating point data type, only one segment can be
-            encoded. Additional segments can be subsequently
+            When `pixel_array` has a floating point data type, only one
+            segment can be encoded. Additional segments can be subsequently
             added to the `Segmentation` instance using the ``add_segments()``
             method.
             If `pixel_array` represents a 3D image, the first dimension
@@ -146,9 +148,10 @@ class Segmentation(SOPClass):
         transfer_syntax_uid: str, optional
             UID of transfer syntax that should be used for encoding of
             data elements. The following lossless compressed transfer syntaxes
-            are supported: JPEG2000 (``"1.2.840.10008.1.2.4.90"``) and
-            JPEG-LS (``"1.2.840.10008.1.2.4.80"``). Lossy compression is not
-            supported.
+            are supported for encapsulated format encoding in case of
+            FRACTIONAL segmentation type:
+            RLE Lossless (``"1.2.840.10008.1.2.5"``) and
+            JPEG 2000 Lossless (``"1.2.840.10008.1.2.4.90"``).
         pixel_measures: PixelMeasures, optional
             Physical spacing of image pixels in `pixel_array`.
             If ``None``, it will be assumed that the segmentation image has the
@@ -218,12 +221,11 @@ class Segmentation(SOPClass):
                 'Only one source image should be provided in case images '
                 'are multi-frame images.'
             )
-
         supported_transfer_syntaxes = {
-            '1.2.840.10008.1.2',       # Implicit Little Endian
-            '1.2.840.10008.1.2.1',     # Explicit Little Endian
-            # '1.2.840.10008.1.2.4.90',  # JPEG2000
-            # '1.2.840.10008.1.2.4.80',  # JPEG-LS
+            ImplicitVRLittleEndian,
+            ExplicitVRLittleEndian,
+            JPEG2000Lossless,
+            RLELossless,
         }
         if transfer_syntax_uid not in supported_transfer_syntaxes:
             raise ValueError(
@@ -241,7 +243,7 @@ class Segmentation(SOPClass):
             series_number=series_number,
             sop_instance_uid=sop_instance_uid,
             instance_number=instance_number,
-            sop_class_uid=SegmentationStorage,
+            sop_class_uid='1.2.840.10008.5.1.4.1.1.66.4',
             manufacturer=manufacturer,
             modality='SEG',
             transfer_syntax_uid=transfer_syntax_uid,
@@ -314,6 +316,12 @@ class Segmentation(SOPClass):
         if self.SegmentationType == SegmentationTypeValues.BINARY.value:
             self.BitsAllocated = 1
             self.HighBit = 0
+            if self.file_meta.TransferSyntaxUID.is_encapsulated:
+                raise ValueError(
+                    'The chosen transfer syntax '
+                    f'{self.file_meta.TransferSyntaxUID} '
+                    'is not compatible with the BINARY segmentation type'
+                )
         elif self.SegmentationType == SegmentationTypeValues.FRACTIONAL.value:
             self.BitsAllocated = 8
             self.HighBit = 7
@@ -365,9 +373,7 @@ class Segmentation(SOPClass):
             # same physical dimensions
             # seg_row_dim = self.Rows * pixel_measures[0].PixelSpacing[0]
             # seg_col_dim = self.Columns * pixel_measures[0].PixelSpacing[1]
-            # src_row_dim = src_img.Rows
-            # Do we need to take ImageOrientationPatient/ImageOrientationPatient
-            # into account?
+            # src_row_dim = src_img.Rows 
 
         if is_multiframe:
             if self._coordinate_system == CoordinateSystemNames.SLIDE:
@@ -375,10 +381,6 @@ class Segmentation(SOPClass):
                     coordinate_system=self._coordinate_system,
                     image_orientation=src_img.ImageOrientationSlide
                 )
-                if src_img.SOPClassUID == VLWholeSlideMicroscopyImageStorage:
-                    self.TotalPixelMatrixRows = src_img.TotalPixelMatrixRows
-                    self.TotalPixelMatrixColumns = \
-                        src_img.TotalPixelMatrixColumns
             else:
                 src_sfg = src_img.SharedFunctionalGroupsSequence[0]
                 source_plane_orientation = src_sfg.PlaneOrientationSequence
@@ -390,7 +392,7 @@ class Segmentation(SOPClass):
         if plane_orientation is None:
             plane_orientation = source_plane_orientation
         self._plane_orientation = plane_orientation
-        self._source_plane_orientation = plane_orientation
+        self._source_plane_orientation = source_plane_orientation
 
         self.DimensionIndexSequence = DimensionIndexSequence(
             coordinate_system=self._coordinate_system
@@ -421,11 +423,11 @@ class Segmentation(SOPClass):
         self.copy_patient_and_study_information(src_img)
 
     def add_segments(
-            self,
-            pixel_array: np.ndarray,
-            segment_descriptions: Sequence[SegmentDescription],
-            plane_positions: Optional[Sequence[PlanePositionSequence]] = None
-        ) -> Dataset:
+        self,
+        pixel_array: np.ndarray,
+        segment_descriptions: Sequence[SegmentDescription],
+        plane_positions: Optional[Sequence[PlanePositionSequence]] = None
+    ) -> Dataset:
         """Adds one or more segments to the segmentation image.
 
         Parameters
@@ -434,20 +436,21 @@ class Segmentation(SOPClass):
             Array of segmentation pixel data of boolean, unsigned integer or
             floating point data type representing a mask image. If `pixel_array`
             is a floating-point array or a binary array (containing only the
-            values ``True`` and ``False`` or ``0`` and ``1``), the segment number
-            used to encode the segment is taken from segment_descriptions.
-            Otherwise, if pixel_array contains multiple integer values, each value
-            is treated as a different segment whose segment number is that integer
-            value. In this case, all segments found in the array must be described
-            in `segment_descriptions`. Note that this is valid for both ``"BINARY"``
-            and ``"FRACTIONAL"`` segmentations.
-            For ``"FRACTIONAL"`` segmentations, values either encode the probability
-            of a given pixel belonging to a segment
+            values ``True`` and ``False`` or ``0`` and ``1``), the segment
+            number used to encode the segment is taken from
+            `segment_descriptions`.
+            Otherwise, if `pixel_array` contains multiple integer values, each
+            value is treated as a different segment whose segment number is
+            that integer value. In this case, all segments found in the array
+            must be described in `segment_descriptions`. Note that this is
+            valid for both ``"BINARY"`` and ``"FRACTIONAL"`` segmentations.
+            For ``"FRACTIONAL"`` segmentations, values either encode the
+            probability of a given pixel belonging to a segment
             (if `fractional_type` is ``"PROBABILITY"``)
             or the extent to which a segment occupies the pixel
             (if `fractional_type` is ``"OCCUPANCY"``).
-            When `pixel_array` has a floating point data type, only one segment can be
-            encoded. Additional segments can be subsequently
+            When `pixel_array` has a floating point data type, only one segment
+            can be encoded. Additional segments can be subsequently
             added to the `Segmentation` instance using the ``add_segments()``
             method.
             If `pixel_array` represents a 3D image, the first dimension
@@ -527,7 +530,7 @@ class Segmentation(SOPClass):
                         'Pixel array contains segments that lack descriptions.'
                     )
 
-        elif (pixel_array.dtype == np.float):
+        elif (pixel_array.dtype in (np.float, np.float32, np.float64)):
             unique_values = np.unique(pixel_array)
             if np.min(unique_values) < 0.0 or np.max(unique_values) > 1.0:
                 raise ValueError(
@@ -568,98 +571,18 @@ class Segmentation(SOPClass):
             # be sure whether there is overlap with the existing segments
             self.SegmentsOverlap = SegmentsOverlapValues.UNDEFINED.value
 
-        src_img = self._source_images[0]
-        is_multiframe = hasattr(src_img, 'NumberOfFrames')
-        if self._coordinate_system == CoordinateSystemNames.SLIDE:
-            if hasattr(src_img, 'PerFrameFunctionalGroupsSequence'):
-                source_plane_positions = [
-                    item.PlanePositionSlideSequence
-                    for item in src_img.PerFrameFunctionalGroupsSequence
-                ]
-            else:
-                # If Dimension Organization Type is TILED_FULL, plane
-                # positions are implicit and need to be computed.
-                image_origin = src_img.TotalPixelMatrixOriginSequence[0]
-                orientation = (
-                    float(src_img.ImageOrientationSlide[0]),
-                    float(src_img.ImageOrientationSlide[1]),
-                    float(src_img.ImageOrientationSlide[2]),
-                    float(src_img.ImageOrientationSlide[3]),
-                    float(src_img.ImageOrientationSlide[4]),
-                    float(src_img.ImageOrientationSlide[5]),
+        src_image = self._source_images[0]
+        is_multiframe = hasattr(src_image, 'NumberOfFrames')
+        if is_multiframe:
+            source_plane_positions, plane_position_values, plane_sort_index = \
+                self.DimensionIndexSequence.get_plane_positions_of_image(
+                    src_image
                 )
-                tiles_per_column = int(
-                    np.ceil(
-                        src_img.TotalPixelMatrixRows /
-                        src_img.Rows
-                    )
-                )
-                tiles_per_row = int(
-                    np.ceil(
-                        src_img.TotalPixelMatrixColumns /
-                        src_img.Columns
-                    )
-                )
-                num_focal_planes = getattr(
-                    src_img,
-                    'NumberOfFocalPlanes',
-                    1
-                )
-                row_range = range(1, tiles_per_column + 1)
-                column_range = range(1, tiles_per_row + 1)
-                depth_range = range(1, num_focal_planes + 1)
-
-                shared_fg = self.SharedFunctionalGroupsSequence[0]
-                pixel_measures = shared_fg.PixelMeasuresSequence[0]
-                pixel_spacing = (
-                    float(pixel_measures.PixelSpacing[0]),
-                    float(pixel_measures.PixelSpacing[1]),
-                )
-                slice_thickness = getattr(
-                    pixel_measures,
-                    'SliceThickness',
-                    1.0
-                )
-                spacing_between_slices = getattr(
-                    pixel_measures,
-                    'SpacingBetweenSlices',
-                    1.0
-                )
-                source_plane_positions = [
-                    compute_plane_positions_tiled_full(
-                        row_index=r,
-                        column_index=c,
-                        depth_index=d,
-                        x_offset=image_origin.XOffsetInSlideCoordinateSystem,
-                        y_offset=image_origin.YOffsetInSlideCoordinateSystem,
-                        z_offset=1.0,  # TODO
-                        rows=self.Rows,
-                        columns=self.Columns,
-                        image_orientation=orientation,
-                        pixel_spacing=pixel_spacing,
-                        slice_thickness=slice_thickness,
-                        spacing_between_slices=spacing_between_slices
-                    )
-                    for r, c, d in itertools.product(
-                        row_range,
-                        column_range,
-                        depth_range
-                    )
-                ]
         else:
-            if is_multiframe:
-                source_plane_positions = [
-                    item.PlanePositionSequence
-                    for item in src_img.PerFrameFunctionalGroupsSequence
-                ]
-            else:
-                source_plane_positions = [
-                    PlanePositionSequence(
-                        coordinate_system=CoordinateSystemNames.PATIENT,
-                        image_position=img.ImagePositionPatient
-                    )
-                    for img in self._source_images
-                ]
+            source_plane_positions, plane_position_values, plane_sort_index = \
+                self.DimensionIndexSequence.get_plane_positions_of_series(
+                    self._source_images
+                )
 
         if plane_positions is None:
             if pixel_array.shape[0] != len(source_plane_positions):
@@ -689,33 +612,6 @@ class Segmentation(SOPClass):
             self._plane_orientation == self._source_plane_orientation
         )
 
-        # For each dimension other than the Referenced Segment Number,
-        # obtain the value of the attribute that the Dimension Index Pointer
-        # points to in the element of the Plane Position Sequence or
-        # Plane Position Slide Sequence.
-        # Per definition, this is the Image Position Patient attribute
-        # in case of the patient coordinate system, or the
-        # X/Y/Z Offset In Slide Coordinate System and the Column/Row
-        # Position in Total Image Pixel Matrix attributes in case of the
-        # the slide coordinate system.
-        plane_position_values = np.array([
-            [
-                np.array(p[0][indexer.DimensionIndexPointer].value)
-                for indexer in self.DimensionIndexSequence[1:]
-            ]
-            for p in plane_positions
-        ])
-
-        # Planes need to be sorted according to the Dimension Index Value
-        # based on the order of the items in the Dimension Index Sequence.
-        # Here we construct an index vector that we can subsequently use to
-        # sort planes before adding them to the Pixel Data element.
-        _, plane_sort_index = np.unique(
-            plane_position_values,
-            axis=0,
-            return_index=True
-        )
-
         # Get unique values of attributes in the Plane Position Sequence or
         # Plane Position Slide Sequence, which define the position of the plane
         # with respect to the three dimensional patient or slide coordinate
@@ -726,32 +622,50 @@ class Segmentation(SOPClass):
             for index in range(plane_position_values.shape[1])
         ]
 
-        # When using binary segmentation type, the previous frames may have been
-        # padded to be a multiple of 8. In this case, we need to decode the
-        # pixel data, add the new pixels and then re-encode. This process
-        # should be avoided if it is not necessary in order to improve
-        # efficiency.
-        if (self.SegmentationType == SegmentationTypeValues.BINARY.value and
-                ((self.Rows * self.Columns * self.SamplesPerPixel) % 8) > 0):
-            re_encode_pixel_data = True
-            logger.warning(
-                'pixel data needs to be re-encoded for binary bitpacking - '
-                'consider using FRACTIONAL instead of BINARY segmentation type'
-            )
-            # If this is the first segment added, the pixel array is empty
-            if hasattr(self, 'PixelData') and len(self.PixelData) > 0:
-                full_pixel_array = self.pixel_array.flatten()
-            else:
-                full_pixel_array = np.array([], np.bool)
-        else:
-            re_encode_pixel_data = False
+        # In certain circumstances, we can add new pixels without unpacking the
+        # previous ones, which is more efficient. This can be done when using
+        # non-encapsulated transfer syntaxes when there is no padding required
+        # for each frame to be a multiple of 8 bits.
+        framewise_encoding = False
+        is_encaps = self.file_meta.TransferSyntaxUID.is_encapsulated
+        if not is_encaps:
+            if self.SegmentationType == SegmentationTypeValues.FRACTIONAL.value:
+                framewise_encoding = True
+            elif self.SegmentationType == SegmentationTypeValues.BINARY.value:
+                # Framewise encoding can only be used if there is no padding
+                # This requires the number of pixels in each frame to be
+                # multiple of 8
+                if (self.Rows * self.Columns * self.SamplesPerPixel) % 8 == 0:
+                    framewise_encoding = True
+                else:
+                    logger.warning(
+                        'pixel data needs to be re-encoded for binary '
+                        'bitpacking - consider using FRACTIONAL instead of '
+                        'BINARY segmentation type'
+                    )
 
+        if framewise_encoding:
             # Before adding new pixel data, remove trailing null padding byte
             if len(self.PixelData) == get_expected_length(self) + 1:
                 self.PixelData = self.PixelData[:-1]
+        else:
+            # In the case of encapsulated transfer syntaxes, we will accumulate
+            # a list of encoded frames to re-encapsulate at the end
+            if is_encaps:
+                if hasattr(self, 'PixelData') and len(self.PixelData) > 0:
+                    # Undo the encapsulation but not the encoding within each
+                    # frame
+                    full_frames_list = decode_data_sequence(self.PixelData)
+                else:
+                    full_frames_list = []
+            else:
+                if hasattr(self, 'PixelData') and len(self.PixelData) > 0:
+                    full_pixel_array = self.pixel_array.flatten()
+                else:
+                    full_pixel_array = np.array([], np.bool)
 
         for i, segment_number in enumerate(described_segment_numbers):
-            if pixel_array.dtype == np.float:
+            if pixel_array.dtype in (np.float, np.float32, np.float64):
                 # Floating-point numbers must be mapped to 8-bit integers in
                 # the range [0, max_fractional_value].
                 planes = np.around(
@@ -764,6 +678,8 @@ class Segmentation(SOPClass):
                 planes[pixel_array == segment_number] = True
             elif pixel_array.dtype == np.bool:
                 planes = pixel_array
+            else:
+                raise TypeError('Pixel array has an invalid data type.')
 
             contained_plane_index = []
             for j in plane_sort_index:
@@ -879,15 +795,23 @@ class Segmentation(SOPClass):
                 self.PerFrameFunctionalGroupsSequence.append(pffp_item)
                 self.NumberOfFrames += 1
 
-            if re_encode_pixel_data:
-                full_pixel_array = np.concatenate([
-                    full_pixel_array,
-                    planes[contained_plane_index].flatten()
-                ])
-            else:
+            if framewise_encoding:
+                # Straightforward concatenation of the binary data
                 self.PixelData += self._encode_pixels(
                     planes[contained_plane_index]
                 )
+            else:
+                if is_encaps:
+                    # Encode this frame and add to the list for encapsulation
+                    # at the end
+                    for f in contained_plane_index:
+                        full_frames_list.append(self._encode_pixels(planes[f]))
+                else:
+                    # Concatenate the 1D array for re-encoding at the end
+                    full_pixel_array = np.concatenate([
+                        full_pixel_array,
+                        planes[contained_plane_index].flatten()
+                    ])
 
             # In case of a tiled Total Pixel Matrix pixel data for the same
             # segment may be added.
@@ -896,8 +820,11 @@ class Segmentation(SOPClass):
             self._segment_inventory.add(segment_number)
 
         # Re-encode the whole pixel array at once if necessary
-        if re_encode_pixel_data:
-            self.PixelData = self._encode_pixels(full_pixel_array)
+        if not framewise_encoding:
+            if is_encaps:
+                self.PixelData = encapsulate(full_frames_list)
+            else:
+                self.PixelData = self._encode_pixels(full_pixel_array)
 
         # Add back the null trailing byte if required
         if len(self.PixelData) % 2 == 1:
@@ -909,16 +836,44 @@ class Segmentation(SOPClass):
         Parameters
         ----------
         planes: numpy.ndarray
-            Array representing one or more segmentation image planes
+            Array representing one or more segmentation image planes.
+            For encapsulated transfer syntaxes, only a single frame may be
+            processed. For other transfer syntaxes, multiple planes in a 3D
+            array may be processed.
 
         Returns
         -------
         bytes
             Encoded pixels
 
+        Raises
+        ------
+        ValueError
+            If multiple frames are passed when using an encapsulated
+            transfer syntax.
+
         """
-        # TODO: compress depending on transfer syntax UID
-        if self.SegmentationType == SegmentationTypeValues.BINARY.value:
-            return pack_bits(planes.flatten())
+        if self.file_meta.TransferSyntaxUID.is_encapsulated:
+            # Check that only a single plane was passed
+            if planes.ndim == 3:
+                if planes.shape[0] == 1:
+                    planes = planes[0, ...]
+                else:
+                    raise ValueError(
+                        'Only single frame can be encoded at at time '
+                        'in case of encapsulated format encoding.'
+                    )
+            return encode_frame(
+                planes,
+                transfer_syntax_uid=self.file_meta.TransferSyntaxUID,
+                bits_allocated=self.BitsAllocated,
+                bits_stored=self.BitsStored,
+                photometric_interpretation=self.PhotometricInterpretation,
+                pixel_representation=self.PixelRepresentation
+            )
         else:
-            return planes.flatten().tobytes()
+            # The array may represent more than one frame item.
+            if self.SegmentationType == SegmentationTypeValues.BINARY.value:
+                return pack_bits(planes.flatten())
+            else:
+                return planes.flatten().tobytes()
